// wordcnt.c - count words, lines, bytes in text files
#include <iostream.h>
#include <fstream.h>
#include <strstream.h>
#include <iomanip.h>

int main(int argc, const char *argv[])
{
	if (argc == 1) {
		cerr << "Usage: " << argv[0] << " filename(s)" << endl;
		return 1;
	}
	int total_bytes = 0;
	int total_lines = 0;
	int total_words = 0;
	int lines = 0;

	for (int i = 1; i < argc; i++) {
		ifstream infile(argv[i], ios::in);		// open file for input
		if (!infile) {									// unable to open
			cerr << "Can't open " << argv[i] << endl;
			continue;									// try next file
		}

		const int Size = 1024;
		char buf[Size];
		char word[Size];
		lines = 0;
		int bytecount = 0;
		int wordcnt = 0;

		while (infile.getline(buf, Size)) {
			bytecount += infile.gcount();			// number of chars read
			lines++;
			istrstream in_mem(buf);					// initialize in_mem
			while (in_mem >> setw(Size) >> word) 		// now count words
				wordcnt++;
		}

		cout << setw(7) << lines << setw(8) << wordcnt 
				<< setw(8) << bytecount << "    " << argv[i] << endl;
		total_bytes += bytecount;
		total_lines += lines;
		total_words += wordcnt;
	}

	if (total_lines != lines) {					// more than one file
		cout << setw(7) << total_lines << setw(8) << total_words
				<< setw(8) << total_bytes << "    Total" << endl;
	}
	return 0;
}

/************************************************************************

$ wordcnt *.C 
    193     925    5873    String.C
     45     140     937    append.C
     32     119     666    ex1.C
     44     146     926    ex2.C
     32      90     621    ex3.C
     59     132     894    ex4.C
     25      60     449    ioprec.C
     36     110     769    readf.C
     57     196    1398    replace.C
    138     532    3265    t1.C
     94     389    2135    t2.C
     37     101     706    t3.C
     29      79     550    t4.C
     20      59     392    ts1.C
     64     245    1658    wordcnt.C
     48     162    1002    writef.C
    953    3485   22241    Total

*/
